@@ -1,10 +1,13 @@
-from enniolearning.utils import FindMaxLength, reject_for_repetition
-from enniolearning.training_fcts import deserialize_score, deserialize_general_note, list_model_files, to_tensor, prepare_sequences, device, reuse_network, get_serialized_model_filename_by_epoch, get_serialized_model_name_by_filename, load_network, parse_epoch
+from enniolearning.utils import FindMaxLength, reject_for_repetition, log
+from enniolearning.training_fcts import deserialize_score, deserialize_general_note, list_model_files, to_tensor, \
+    prepare_sequences, device, reuse_network, get_serialized_model_filename_by_epoch, \
+    get_serialized_model_name_by_filename, load_network, parse_epoch
 import numpy as np
 import matplotlib.pyplot as plt
 import pickle
 from enniolearning.eta_tools import EtaPrinter, get_progress_ratio
 from music21.note import Rest
+import logging
 
 SKIP_LOSS_FUNCTION = 'Skip Loss Function'
 SKIP_LEARNING_RATE = 'Skip Learning Rate'
@@ -14,7 +17,6 @@ STATUS_TYPE_ETA = 'eta'
 
 def evaluate_generated_score(prediction_by_track, network_input, enable_evaluation=True,
                              repetition_by_track_ratio_accepted=0.4, **kwargs):
-
     if not enable_evaluation:
         return 0, False
 
@@ -27,7 +29,7 @@ def evaluate_generated_score(prediction_by_track, network_input, enable_evaluati
     plagia_part = 0
     for track_idx in range(flatten_input.shape[1]):
         plagia_part = max(plagia_part, FindMaxLength(flatten_input[:][track_idx], prediction_by_track[track_idx]))
-    #     print(f"Evaluation : morceau généré à {100 * (len(prediction) - plagia_part)/len(prediction)}%")
+    #     log(f"Evaluation : morceau généré à {100 * (len(prediction) - plagia_part)/len(prediction)}%", **kwargs)
 
     # Repetition detection : not pleasant to here
     rejected_by_track = np.array(
@@ -40,8 +42,8 @@ def evaluate_generated_score(prediction_by_track, network_input, enable_evaluati
 
 def create_generation_input(previous_prediction_output_as_int=None, network_input=None, **kwargs):
     if previous_prediction_output_as_int is not None:
-        print('From previous prediction')
-        return previous_prediction_output_as_int # shape=(generate_output_length, nb_instruments)
+        log('From previous prediction', **kwargs)
+        return previous_prediction_output_as_int  # shape=(generate_output_length, nb_instruments)
     if network_input is not None:
         # pick a random sequence from the input as a starting point for the prediction
         start = np.random.randint(0, len(network_input) - 1)
@@ -49,7 +51,7 @@ def create_generation_input(previous_prediction_output_as_int=None, network_inpu
     raise KeyError('Must provide at least one of [previous_prediction_output_as_int, network_input]')
 
 
-def generate_notes(model, int_to_note, n_vocab,  generate_output_length=50, **kwargs):
+def generate_notes(model, int_to_note, n_vocab, generate_output_length=50, **kwargs):
     """ Generate notes from the neural network based on a sequence of notes.
     Dim 0 = time sequence (generate_output_length) ; Dim 1 = instrument index.
     """
@@ -58,14 +60,16 @@ def generate_notes(model, int_to_note, n_vocab,  generate_output_length=50, **kw
     prediction_output_as_int = []
     # generate generate_output_length notes
     for note_index in range(generate_output_length):
-        #         print('pattern', pattern)
-        prediction_input = np.reshape(pattern, (1, pattern.shape[0], pattern.shape[1]))  # shape (1,sequence_length,len(instrument_index))
+        #         log('pattern', pattern, **kwargs)
+        prediction_input = np.reshape(pattern, (
+        1, pattern.shape[0], pattern.shape[1]))  # shape (1,sequence_length,len(instrument_index))
         prediction_input = prediction_input / float(n_vocab)
         prediction_input = to_tensor(prediction_input)
         prediction = model(prediction_input)  # shape (1,n_vocab,len(instrument_index)) (=len(int_to_note))
         if prediction.shape[0] != 1:
-            print(
-                f"expected batch output size == 1, was {prediction.shape[0]}")  # could it be some feature to return several sequence in a batch (batch_size > 1) -> check perf
+            log(
+                f"expected batch output size == 1, was {prediction.shape[0]}", level=logging.WARN,
+                **kwargs)  # could it be some feature to return several sequence in a batch (batch_size > 1) -> check perf
         prediction = prediction.squeeze(0)  # ignore 1st dimension (batch index)
         next_note_as_int = []
         next_note = []
@@ -95,10 +99,11 @@ def create_midi(prediction_output, instruments_index,
         midi_stream.show('midi')
     if create_midi_file:
         midi_stream.write('midi', fp=output_filename)
-        # print(f"File {output_filename} created !")
+        log(f"File {output_filename} created !", level=logging.DEBUG, **kwargs)
         return output_filename
     return None
 
+
 def is_a_rest(general_note, **kwargs):
     # TRY perf ? return general_note == PADDING_STR or 'R_' in general_note
     return type(deserialize_general_note(general_note, **kwargs)) is Rest
@@ -121,18 +126,19 @@ def ignore_few_note(prediction, i, min_no_rest_sequence_accepted, is_rest=False)
     [1] nb of analyzed indices, so next iteration do not have to process them again
     """
     if is_rest:
-        return False, 1 # N/A
+        return False, 1  # N/A
     if min_no_rest_sequence_accepted > 1:
-        for j in range(1, min_no_rest_sequence_accepted): # because i is already known as non-rest
+        for j in range(1, min_no_rest_sequence_accepted):  # because i is already known as non-rest
             index_to_analyze = i + j
             if index_to_analyze < len(prediction):
                 if is_all_instrument_rest(prediction[index_to_analyze]):
                     return True, j
-    return False, min(min_no_rest_sequence_accepted, len(prediction)-i)
+    return False, min(min_no_rest_sequence_accepted, len(prediction) - i)
 
 
 def post_processing(prediction_output, prediction_output_as_int, enable_post_processing=True,
-                    min_no_rest_sequence_accepted=3, max_rest_sequence_accepted=10, serialize_prediction=False, **kwargs):
+                    min_no_rest_sequence_accepted=3, max_rest_sequence_accepted=10, serialize_prediction=False,
+                    **kwargs):
     """
     Performs tasks :
 
@@ -158,30 +164,31 @@ def post_processing(prediction_output, prediction_output_as_int, enable_post_pro
     rest_step_cumul = []
     should_ignore_note = []
     i = 0
-    while i < generate_output_length :
+    while i < generate_output_length:
         is_rest = is_all_instrument_rest(prediction_output[i], **kwargs)
         # detect isolated notes only if inside a long rest
         is_isolated_note, step = (False, 1) if rest_sequence_len <= max_rest_sequence_accepted \
             else ignore_few_note(prediction_output, i, min_no_rest_sequence_accepted, is_rest)
-        rest_sequence_len = rest_sequence_len+1 if is_rest or is_isolated_note else 0
+        rest_sequence_len = rest_sequence_len + 1 if is_rest or is_isolated_note else 0
         if not is_isolated_note:
             rest_step_cumul.extend(rest_sequence_len for _ in range(step))
             should_ignore_note.extend(False for _ in range(step))
             i += step
         else:
             rest_step_cumul.append(rest_sequence_len)
-            should_ignore_note.append(is_isolated_note) # True -> prediction will be skipped
+            should_ignore_note.append(is_isolated_note)  # True -> prediction will be skipped
             i += 1
     shrink_long_rest_mask = np.array(rest_step_cumul) <= max_rest_sequence_accepted
     should_ignore_note = np.array(should_ignore_note)
     post_processing_indexing = np.logical_and(shrink_long_rest_mask, np.logical_not(should_ignore_note))
-    post_processing_score = np.sum(post_processing_indexing) / generate_output_length # kept predicted notes ratio
+    post_processing_score = np.sum(post_processing_indexing) / generate_output_length  # kept predicted notes ratio
 
     if serialize_prediction and type(serialize_prediction) is str:
         with open(f"{serialize_prediction}_postprocessed", 'xb') as filepath:
             pickle.dump(prediction_output_as_int[post_processing_indexing], filepath)
-
-    return prediction_output[post_processing_indexing], prediction_output_as_int[post_processing_indexing], post_processing_score
+    log(f"Prediction post processed", level=logging.DEBUG, **kwargs)
+    return prediction_output[post_processing_indexing], prediction_output_as_int[
+        post_processing_indexing], post_processing_score
 
 
 def generate(notes=None, instruments_index=None, model=None, batch_size=1, network_input=None, int_to_note=None,
@@ -193,23 +200,23 @@ def generate(notes=None, instruments_index=None, model=None, batch_size=1, netwo
 
     lstm_input_size = len(instruments_index)
 
-    if network_input is None or int_to_note is None :
+    if network_input is None or int_to_note is None:
         network_input, _, note_to_int, int_to_note, sequence_length = prepare_sequences(notes,
                                                                                         batch_size=batch_size,
                                                                                         predict_mode=True, **kwargs)
     n_vocab = len(int_to_note)
 
     model = reuse_network(model, batch_size=batch_size, n_vocab=n_vocab, input_size=lstm_input_size, **kwargs)
+    log(f"Model loaded {str(model)}", level=logging.DEBUG, **kwargs)
     prediction_output, prediction_output_as_int = generate_notes(model,
                                                                  int_to_note,
                                                                  n_vocab,
                                                                  network_input=network_input,
                                                                  **kwargs)
-
+    log(f"{len(prediction_output)} notes predicted", level=logging.DEBUG, **kwargs)
     prediction_output, prediction_output_as_int, post_processing_score = post_processing(prediction_output,
                                                                                          prediction_output_as_int,
                                                                                          **kwargs)
-
     prediction_output_by_instru = prediction_output.transpose()
 
     # evaluate prediction
@@ -227,16 +234,17 @@ def generate(notes=None, instruments_index=None, model=None, batch_size=1, netwo
 
     output_filename = create_midi(prediction_output_by_instru, instruments_index, **kwargs)
 
-    return output_filename, prediction_output_as_int, (diversity_rate, plagia_part, rejected_for_repetition, post_processing_score)
+    return output_filename, prediction_output_as_int, (
+    diversity_rate, plagia_part, rejected_for_repetition, post_processing_score)
 
 
 def or_load_from_file(notes=None, instruments_index=None, dump_path='data', **kwargs):
     if notes is None:
         # load the notes used to train the model
-        with open(dump_path+'/notes_by_instrument', 'rb') as filepath:
+        with open(dump_path + '/notes_by_instrument', 'rb') as filepath:
             notes = pickle.load(filepath)
     if instruments_index is None:
-        with open(dump_path+'/instruments_index', 'rb') as filepath:
+        with open(dump_path + '/instruments_index', 'rb') as filepath:
             instruments_index = pickle.load(filepath)
     return notes, instruments_index
 
@@ -246,7 +254,8 @@ LEARNING_RATE = '{1}'
 
 
 def evaluate_models(path, exec_nb=1, loss_func_names=['BCELoss()'], lrs=[1e-3, 1e-4, 1e-5], select_epochs=False,
-                    generate_output_length=300, print_progress=True, notes=None, instruments_index=None, batch_size=1, jordan_instance=None, **kwargs):
+                    generate_output_length=300, print_progress=True, log_progress=False, notes=None,
+                    instruments_index=None, batch_size=1, jordan_instance=None, **kwargs):
     """Path : use make_path()
     use '{0}' or LOSS_FUNCTION for loss function name
     use '{1}' or LEARNING_RATE" for learning rate
@@ -268,8 +277,8 @@ def evaluate_models(path, exec_nb=1, loss_func_names=['BCELoss()'], lrs=[1e-3, 1
     lstm_input_size = len(instruments_index)
 
     _, normalized_input, note_to_int, int_to_note, sequence_length = prepare_sequences(notes,
-                                                                                                   batch_size=batch_size,
-                                                                                                   predict_mode=True)
+                                                                                       batch_size=batch_size,
+                                                                                       predict_mode=True)
     n_vocab = len(note_to_int)
 
     evaluation = {}
@@ -288,10 +297,10 @@ def evaluate_models(path, exec_nb=1, loss_func_names=['BCELoss()'], lrs=[1e-3, 1
                 break
 
             subfolder = path.format(loss_func_name, f"{lr:.0E}")
-            #             print(f"In folder {subfolder}")
+            # log(f"In folder {subfolder}", level=logging.DEBUG, **kwargs)
             evaluation[subfolder] = {}
 
-            model_files_in_folder = list_model_files(subfolder)
+            model_files_in_folder = list_model_files(subfolder, **kwargs)
             model_files = select_models(model_files_in_folder, select_epochs)
             for model_passed, model_filename in enumerate(model_files):
 
@@ -322,18 +331,21 @@ def evaluate_models(path, exec_nb=1, loss_func_names=['BCELoss()'], lrs=[1e-3, 1
                             interrupt_loop = False
                         break
 
-                    if print_progress:
-                        progress_tuples = (loss_function_passed, len(loss_func_names)),\
-                                          (lr_passed, len(lrs)),\
-                                          (model_passed, len(model_files)),\
+                    if print_progress or log_progress:
+                        progress_tuples = (loss_function_passed, len(loss_func_names)), \
+                                          (lr_passed, len(lrs)), \
+                                          (model_passed, len(model_files)), \
                                           (i, exec_nb)
                         eta_as_string = eta.print_eta(*progress_tuples)
                         general_progress = f"Loss function = {loss_func_name} ({get_percent(loss_function_passed + 1, len(loss_func_names))})," \
-                                   f" LR = {lr} ({get_percent(lr_passed + 1, len(lrs))})," \
-                                   f" model filename = {model_filename} ({get_percent(model_passed + 1, len(model_files))})," \
-                                   f" execution = {i + 1} / {exec_nb}." \
-                                   f" ETA : {eta_as_string}."
-                        print(general_progress, end='\r', flush=True)
+                                           f" LR = {lr} ({get_percent(lr_passed + 1, len(lrs))})," \
+                                           f" model filename = {model_filename} ({get_percent(model_passed + 1, len(model_files))})," \
+                                           f" execution = {i + 1} / {exec_nb}." \
+                                           f" ETA : {eta_as_string}."
+                        if print_progress:
+                            print(general_progress, end='\r', flush=True)
+                        if log_progress:
+                            log(general_progress, **kwargs)
                         if jordan_instance:
                             progress = to_progress(get_progress_ratio(*progress_tuples))
                             jordan_instance.send_status(general_progress)
@@ -341,7 +353,7 @@ def evaluate_models(path, exec_nb=1, loss_func_names=['BCELoss()'], lrs=[1e-3, 1
                             jordan_instance.send_typed_status(status_type=STATUS_TYPE_ETA, status=eta_as_string)
 
                     _, _, \
-                    (diversity_rate, plagia_part, rejected_for_repetition, post_processing_score) =\
+                    (diversity_rate, plagia_part, rejected_for_repetition, post_processing_score) = \
                         generate(notes, instruments_index,
                                  model,
                                  generate_output_length=generate_output_length,
@@ -350,7 +362,8 @@ def evaluate_models(path, exec_nb=1, loss_func_names=['BCELoss()'], lrs=[1e-3, 1
                                  int_to_note=int_to_note,
                                  **kwargs)
                     if jordan_instance:
-                        jordan_instance.send_status(f"Diversity rate : {diversity_rate}. Plagia part : {plagia_part}. Rejected for repetition : {rejected_for_repetition}.")
+                        jordan_instance.send_status(
+                            f"Diversity rate : {diversity_rate}. Plagia part : {plagia_part}. Rejected for repetition : {rejected_for_repetition}.")
 
                         jordan_message = jordan_instance.read_message()
                         if jordan_message:
@@ -379,13 +392,12 @@ def evaluate_models(path, exec_nb=1, loss_func_names=['BCELoss()'], lrs=[1e-3, 1
 
 
 def to_progress(ratio, base=100.0, as_int=True):
-    progress = ratio*base
+    progress = ratio * base
     if as_int:
         return int(progress)
     return progress
 
 
-
 def select_models(files, select):
     if select:
         select_files = []
@@ -414,20 +426,20 @@ def extract_evaluation_feature(dic, first_key, third_key):
     return extract
 
 
-def extract_epoch_from_filename(dic, first_key):
+def extract_epoch_from_filename(dic, first_key, **kwargs):
     """dic from evaluate_models()
     first_key is folder
     """
     extract = []
     for model_filename, model_stats in dic[first_key].items():
-        extract.append(parse_epoch(model_filename))
+        extract.append(parse_epoch(model_filename, **kwargs))
     return extract
 
 
 def plot_model_evaluation(evaluation, feature, **kwargs):
     plt.figure(figsize=(16, 10))
     for model_type in evaluation.keys():
-        plt.plot(extract_epoch_from_filename(evaluation, model_type),
+        plt.plot(extract_epoch_from_filename(evaluation, model_type, **kwargs),
                  extract_evaluation_feature(evaluation, model_type, feature), label=model_type)
     #     plt.title(feature)
     plt.xlabel("Epoch / Model")
