@@ -14,10 +14,11 @@ from music21 import *
 from torch import nn
 from torch.utils.data import DataLoader
 from torch.utils.data import TensorDataset
+import logging
 
 from enniolearning.ennio_midi_model import ModelMidiLstm
 from enniolearning.eta_tools import EtaPrinter
-from enniolearning.utils import to_categorical, label_to_int
+from enniolearning.utils import to_categorical, label_to_int, log
 
 #https://stackoverflow.com/questions/49939275/python-music21-library-create-png-from-stream
 #environment.set("musescoreDirectPNGPath", "/usr/bin/musescore")
@@ -42,7 +43,7 @@ def get_notes_by_instrument(midi_files_path="data/midi_songs/*.mid", apply_part_
             and os.path.isfile(dump_path+'/notes_by_instrument')\
             and os.path.isfile(dump_path+'/instruments_index'):
 
-        print('Reusing serialized notes from previous parsing')
+        log('Reusing serialized notes from previous parsing', **kwargs)
         with open(dump_path + '/notes_by_instrument', 'rb') as filepath:
             notes_by_instrument = pickle.load(filepath)
         with open(dump_path + '/instruments_index', 'rb') as filepath:
@@ -57,11 +58,11 @@ def get_notes_by_instrument(midi_files_path="data/midi_songs/*.mid", apply_part_
         if len(files_to_parse) is 0:
             raise FileNotFoundError(f'No file to parse in {midi_files_path}. Please check [midi_files_path] key.')
         for file in files_to_parse:
-            print("Parsing %s" % file)
+            log(f"Parsing {file}", **kwargs)
             try:
                 midi = converter.parse(file)
             except UnicodeDecodeError:
-                print(" ! Enconding error on %s" % file)
+                log(f"Enconding error on {file}", **kwargs)
                 continue
 
             notes_for_this_song, instruments_index = get_notes_by_instrument_for_score(midi, instruments_index, **kwargs)
@@ -75,7 +76,7 @@ def get_notes_by_instrument(midi_files_path="data/midi_songs/*.mid", apply_part_
         notes_by_instrument, songs_length = format_notes_to_sequence(notes_by_instrument, instruments_index, songs_quarterLength, **kwargs)
 
         if plot_raw_songs:
-            plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_length, **kwargs)
+            plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_length, 'Raw Dataset', **kwargs)
 
         # 2/ skip 2nd dim, as if dataset was a single big song
         notes_by_instrument = flatten_skip_song(notes_by_instrument)
@@ -85,7 +86,7 @@ def get_notes_by_instrument(midi_files_path="data/midi_songs/*.mid", apply_part_
             notes_by_instrument, instruments_index = filter_parts(notes_by_instrument, instruments_index, **kwargs)
 
         if plot_dataset:
-            plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_length=songs_length, **kwargs)
+            plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_length, 'Final Dataset', **kwargs)
 
         with open(dump_path+'/notes_by_instrument', 'wb') as filepath:
             pickle.dump(notes_by_instrument, filepath)
@@ -95,7 +96,7 @@ def get_notes_by_instrument(midi_files_path="data/midi_songs/*.mid", apply_part_
     return notes_by_instrument, instruments_index
 
 
-def plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_length, **kwargs):
+def plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_length, title: str = '', **kwargs):
     nb_songs = len(files_to_parse)
     is_song_dimension = notes_by_instrument.shape[1] == nb_songs
     if is_song_dimension:  # before flatten
@@ -103,7 +104,7 @@ def plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_leng
     else:  # no more song distinction, they are flattened
         pitchnames = set(note for part in notes_by_instrument for note in part)
     note_to_int = dict((note, number) for number, note in enumerate(pitchnames))
-    fig, ax = plt.subplots(nb_songs)
+    fig, ax = plt.subplots(nb_songs, num=title)
     fig.tight_layout()
     from_index = 0
     for song_index in range(nb_songs):  # by song
@@ -118,6 +119,7 @@ def plot_song(files_to_parse, notes_by_instrument, instruments_index, songs_leng
 
         ax[song_index].imshow(notes_by_instrument_as_number(song_notes, note_to_int), aspect='auto')
 
+
 def notes_by_instrument_as_number(notes_by_instrument, note_to_int, **kwargs):
     return np.array([[note_to_int[notes_by_instrument[i][j]] for j in range(len(notes_by_instrument[i]))] for i in range(len(notes_by_instrument))])
 
@@ -130,7 +132,7 @@ def filter_parts(notes_by_instrument_detailed, instruments_index, **kwargs):
         indices_of_same_instrument = list(i_idx for i_idx, is_cat in enumerate(instru_index_mask) if is_cat)
         #         notes_of_macro_instrument = []
         notes_for_same_instrument, same_instruments_index = filter_algo(
-            notes_by_instrument_detailed[indices_of_same_instrument], instruments_index[indices_of_same_instrument])
+            notes_by_instrument_detailed[indices_of_same_instrument], instruments_index[indices_of_same_instrument], **kwargs)
 
         filtered_notes_by_instrument.extend(notes_for_same_instrument)
         filtered_instruments_index.extend(same_instruments_index)
@@ -138,7 +140,7 @@ def filter_parts(notes_by_instrument_detailed, instruments_index, **kwargs):
     return np.array(filtered_notes_by_instrument), np.array(filtered_instruments_index)
 
 
-def filter_algo(notes_by_instrument, instruments_index):
+def filter_algo(notes_by_instrument, instruments_index, **kwargs):
     # may add several filters, one after another, with global_filter = filter1 & filter2 & filter3 (when filterX is a np.array)
 
     diversity_by_instru = {}
@@ -153,8 +155,8 @@ def filter_algo(notes_by_instrument, instruments_index):
     diversity_filter_mask = diversity_values >= min(diversity_values.max() - diversity_values.max() * 0.5,
                                                     np.quantile(diversity_values, 0.85, interpolation='higher'))
 
-    print(
-        f"Diversity filter : keep {instruments_index[diversity_filter_mask]}, reject {instruments_index[~diversity_filter_mask]}")
+    log(
+        f"Diversity filter : keep {instruments_index[diversity_filter_mask]}, reject {instruments_index[~diversity_filter_mask]}", **kwargs)
 
     return notes_by_instrument[diversity_filter_mask], instruments_index[diversity_filter_mask]
 
@@ -202,7 +204,7 @@ def get_notes_by_instrument_for_score(score, instruments_index : list, **kwargs)
     notes_for_this_song = {}
     parts = extract_parts(score, **kwargs)
     for instru, p in parts.items():
-        #         print('Reading notes of part', instru)
+        #         log(f'Reading notes of part {instru}', level=logging.DEBUG, **kwargs)
         can_merge, instru_index = instrument_can_be_merged_with_index(instru, instruments_index, all_parts=parts, **kwargs)
         if not can_merge:
             instruments_index.append(instru)
@@ -221,7 +223,7 @@ def get_notes_by_instrument_for_score(score, instruments_index : list, **kwargs)
             elif isGeneralNote(element):
                 notes_for_this_song[instru_index].extend(serialize_general_note(p, element_index, **kwargs))
             else:
-                print(f"Unhandled type of element in part : {type(element)}")
+                log(f"Unhandled type of element in part : {type(element)}", level=logging.WARN, **kwargs)
     return notes_for_this_song, instruments_index
 
 
@@ -325,7 +327,8 @@ def deserialize_part(serialized_part, **kwargs):
     # create note and chord objects based on the values generated by the model
     for pattern in serialized_part:
         new_note = deserialize_general_note(pattern, **kwargs)
-        part.append(new_note)
+        if new_note is not None:
+            part.append(new_note)
     return part
 
 
@@ -344,7 +347,8 @@ def deserialize_general_note(pattern, **kwargs):
         general_note.quarterLength = quarter_length
         return general_note
     else:
-        print(f"Unhandled deserialization for note {pattern}")
+        log(f"Unhandled deserialization for note {pattern}", level=logging.ERROR, **kwargs)
+        return None
 
 
 def padding_note(padding_rest_quarter_length=1, **kwargs):
@@ -370,7 +374,7 @@ def deserialize_note(pattern):
 
 def extract_quarter_length(pattern, min_length=1/3, **kwargs):
     if float(pattern) <= 0.0:
-        print("cheat")
+        log("cheating on note quarter length", level=logging.WARN, **kwargs)
         return Fraction(min_length)
     # can be a decimal (e.g 0.5) or a fraction (e.g 1/3)
     return Fraction(pattern)
@@ -396,20 +400,21 @@ def serialize_general_note(s, element_index=0, split_long_note=True, min_length=
     max_length = max_length_rest if isinstance(element, note.Rest) else max_length_note
 
     note_length = extract_length(s, element_index, **kwargs)
-    pitch_as_str = serialize_pitch(element)
+    pitch_as_str = serialize_pitch(element, **kwargs)
 
     # work by measure here
     # instead of having a very long note/rest, add many notes/rests, each having max quarterLength=4
     array_of_serialized_notes = []
-    if split_long_note and note_length/max_length > 1.0:
-        nb_max_length_notes = int(note_length / max_length)
-        for i in range(nb_max_length_notes):
-            array_of_serialized_notes.append(as_serialized_note(pitch_as_str, max_length, min_length=min_length, **kwargs))
-        surplus = note_length % max_length
-        if surplus >= min_length:
-            array_of_serialized_notes.append(as_serialized_note(pitch_as_str, surplus, min_length=min_length, **kwargs))
-    else:
-        array_of_serialized_notes.append(as_serialized_note(pitch_as_str, note_length, min_length=min_length, **kwargs))
+    if pitch_as_str is not None:
+        if split_long_note and note_length/max_length > 1.0:
+            nb_max_length_notes = int(note_length / max_length)
+            for i in range(nb_max_length_notes):
+                array_of_serialized_notes.append(as_serialized_note(pitch_as_str, max_length, min_length=min_length, **kwargs))
+            surplus = note_length % max_length
+            if surplus >= min_length:
+                array_of_serialized_notes.append(as_serialized_note(pitch_as_str, surplus, min_length=min_length, **kwargs))
+        else:
+            array_of_serialized_notes.append(as_serialized_note(pitch_as_str, note_length, min_length=min_length, **kwargs))
 
     return array_of_serialized_notes
 
@@ -418,17 +423,18 @@ def as_serialized_note(pitch_str, length, **kwargs):
     return pitch_str + '_' + str(length_rounding(length, **kwargs))
 
 
-def serialize_pitch(element):
+def serialize_pitch(element, **kwargs):
     if isinstance(element, note.Note):
         return str(element.nameWithOctave)
     elif isinstance(element, chord.Chord):
-        return '.'.join(serialize_pitch(p) for p in element.pitches)
+        return '.'.join(serialize_pitch(p, **kwargs) for p in element.pitches)
     elif isinstance(element, note.Rest):
         return 'R'
     elif isinstance(element, pitch.Pitch):
         return str(element.nameWithOctave)
     else:
-        print(f"Unhandled type of Note in part : {type(element)}")
+        log(f"Unhandled type of Note in part : {type(element)}", level=logging.ERROR, **kwargs)
+        return None
 
 
 def extract_length(s, element_index, **kwargs):
@@ -499,7 +505,7 @@ def extract_details_on_instrument(p, voice=None, history=[], **kwargs):
     if instru_name in history:
         instru_name = increment_instru_name(instru_name, 2, history)
     history.append(instru_name)
-    #     print(instru_name, type(instru_name))
+    #     log(f"instru_name {type(instru_name)}", level=logging.DEBUG, **kwargs)
     return instru_name
 
 
@@ -611,7 +617,7 @@ def best_instrument(*guesses, **kwargs):
     better_name = best_instrument_name(best_candidate, instrument_guesses, **kwargs)
     if better_name:
         best_candidate.instrumentName = better_name
-    #     print(f"best candidate : {best_candidate.instrumentName} of type {type(best_candidate)} from {guesses}")
+    #     log(f"best candidate : {best_candidate.instrumentName} of type {type(best_candidate)} from {guesses}", level=logging.DEBUG, **kwargs)
     return best_candidate
 
 
@@ -783,7 +789,7 @@ def load_network(model=None, batch_size=1, device=device, weight_file=None, **kw
         model = create_network(batch_size=batch_size, device=device, **kwargs)
 
         if weight_file is not None:
-            print(f'Loading weight from {weight_file}')
+            log(f'Loading weight from {weight_file}', **kwargs)
             if type(weight_file) is str:
                 model.load_state_dict(torch.load(weight_file))
     return model
@@ -797,7 +803,7 @@ def resume_or_create_model(resume_model=None, lr=None, test_name=None, **kwargs)
             filename = resume_model
         else:
             raise KeyError(f'resume model {resume_model} of type {type(resume_model)} is not supported to resume training from it')
-        print(f"Resuming training from filename {filename}")
+        log(f"Resuming model from filename {filename}", **kwargs)
         model = load_network(filename, **kwargs)
     else:
         model = create_network(**kwargs)
@@ -826,11 +832,11 @@ def loss_batch(model, criterion, batch_input, batch_target, opt=None, **kwargs):
     if opt is not None:
         opt.zero_grad()
 
-    #     print(f"input : {xb.shape}")
+    #     log(f"input : {xb.shape}", level=logging.DEBUG)
     predictions = model(batch_input)
-    #     print(f"Computing loss between")
-    #     print(f"output : {predictions.shape}")
-    #     print(f"target : {batch_target.shape}")
+    #     log(f"Computing loss between", level=logging.DEBUG, **kwargs)
+    #     log(f"output : {predictions.shape}", level=logging.DEBUG, **kwargs)
+    #     log(f"target : {batch_target.shape}", level=logging.DEBUG, **kwargs)
     #     predictions[some_batch] is 2D tensor, len=158=n_vocab. probabilities of the next note/pitch
     #     batch_target[some_batch] is 1D tensor, len=158=n_vocab. one 1, n_vocab-1 0 -> next note/pitch target
     loss = criterion(predictions, batch_target.float())
@@ -863,8 +869,8 @@ def fit(model, loss_func, opt, batch_size, epochs=50, train_dl=None, print_epoch
         model.train()
         #         model.zero_grad() #right ????
         for xb, yb in train_dl:
-            #             print(f"xb shape : {xb.shape}") #(8, 100, 1) = (batch_size, time_sequence_length, 1 note/pitch)
-            #             print(f"yb shape : {yb.shape}") #(8, 158) = (batch_size, n_vocab)
+            #             log(f"xb shape : {xb.shape}", level=logging.DEBUG, **kwargs)  # (batch_size, time_sequence_length, 1 note/pitch)
+            #             log(f"yb shape : {yb.shape}", level=logging.DEBUG, **kwargs)  # (batch_size, n_vocab)
             loss = loss_batch(model, loss_func, xb, yb, opt, **kwargs)
             epoch_loss += loss
 
@@ -882,8 +888,8 @@ def fit(model, loss_func, opt, batch_size, epochs=50, train_dl=None, print_epoch
             epoch_hrs, epoch_mins, epoch_secs = epoch_time(start_time, end_time)
             epoch_status = f'Epoch: {epoch + 1:02} | Epoch Time: {epoch_hrs}h {epoch_mins}m {epoch_secs}s'
             train_loss_status = f'\tTrain Loss: {epoch_loss_train:.5f}'
-            print(epoch_status)
-            print(train_loss_status)
+            log(epoch_status, **kwargs)
+            log(train_loss_status, **kwargs)
             if jordan_instance:
                 jordan_instance.send_status(epoch_status + train_loss_status)
 
@@ -932,14 +938,14 @@ def get_serialized_model_name_by_filename(subfolder, filename, save_folder_root=
     return save_folder_root + subfolder + filename
 
 
-def parse_epoch(filename):
+def parse_epoch(filename, **kwargs):
     regex = ".*" + MODEL_FILE_PREFIX + "([0-9]{3})"
     try:
         parsed = re.search(regex, filename)
         if parsed and parsed.group(1):
             return int(parsed.group(1))
     except:
-        print("parsing error on filename", filename)
+        log(f"parsing error on filename {filename}", level=logging.FATAL, **kwargs)
     return filename
 
 
@@ -952,11 +958,11 @@ def make_path(*folders):
     return separator.join(folders) + separator
 
 
-def list_model_files(subfolder):
+def list_model_files(subfolder, **kwargs):
     try:
         return os.listdir(DEFAULT_SAVE_FOLDER + subfolder)
     except FileNotFoundError as fnf:
-        print('error', 'File not found', str(fnf))
+        log(f'File not found {str(fnf)}', level=logging.ERROR, **kwargs)
         return []
 
 
@@ -973,7 +979,7 @@ def pad_instru_part(notes_seq, size, quarterLength, **kwargs):
     if len(notes_seq) >= size:
         return notes_seq
 
-    print('*** IMPROVE THIS ? *** because we may loose time and synchronization between instruments here')
+    log('*** IMPROVE THIS ? *** because we may loose time and synchronization between instruments here', level=logging.DEBUG, **kwargs)
     padding = np.full(size - len(notes_seq), PADDING_STR)
     return np.hstack((notes_seq, padding))
 
@@ -1114,7 +1120,7 @@ def train_with_losses(loss_funcs=None, learning_rates=None,
             loss_func = loss_func.to(device)
 
             start_training_msg = f"Train model with loss {loss_func} with learning rate {lr}"
-            print(start_training_msg)
+            log(start_training_msg, **kwargs)
             if jordan_instance:
                 jordan_instance.send_status(start_training_msg)
                 if jordan_loss_function_lr_task:
@@ -1150,11 +1156,5 @@ def train_with_losses(loss_funcs=None, learning_rates=None,
 
             if jordan_loss_function_lr_task:
                 jordan_loss_function_lr_task.complete()
-    #             except:
-    #                 print("error", sys.exc_info()[0])
-    #                 raise ValueError("error", sys.exc_info()[0]).with_traceback(sys.exc_info()[2])
-    #                 continue
-
-    # model.cpu() del model
 
     return train_loss
\ No newline at end of file
